#!/usr/bin/python
# -*- encoding: utf-8 -*-

# cteni a zapis casu do RTC obvodu PCF8583 


import smbus
import datetime
import os
import sys

adr=0x51    # adresa obvodu, kdyz je adresovaci noha v "1"
#adr=0x50    # adresa obvodu, kdyz je adresovaci noha v "0"


bus = smbus.SMBus(1)  # novejsi varianta RasPi (512MB)

try:
  bus.write_byte_data(adr,0x0e,0b00000000)  # Nastaveni na rezim casu s krystalem 32,768kHz
except: 
  print "CHYBA - RTC obvod nekomunikuje"
  exit(0)


#=============================================
# podprogram pro zjisteni casu v obvodu RTC a jeho zobrazeni, nebo nastaveni casu v RasPi podle RTC
def rtc_info(nastav_cas=False):

  datcas = datetime.datetime.utcnow()   # datcas bude obsahovat aktualni datum a cas v UTC (GMT)
  print "Systemovy cas v RasPi :" , datcas , "UTC"

  nazevdne = [ '???????', 'pondeli' , 'utery  ' , 'streda ' , 'ctvrtek' , 'patek  ' , 'sobota ' , 'nedele ' ]


  rtc =  bus.read_i2c_block_data(adr,0x00)

  ses = ((rtc[0x01] & 0b11110000)>>4) *10 + (rtc[0x01] & 0b00001111)   # setiny sekundy
  sek = ((rtc[0x02] & 0b01110000)>>4) *10 + (rtc[0x02] & 0b00001111)   # sekundy
  min = ((rtc[0x03] & 0b01110000)>>4) *10 + (rtc[0x03] & 0b00001111)   # minuty
  hod = ((rtc[0x04] & 0b00110000)>>4) *10 + (rtc[0x04] & 0b00001111)   # hodiny

  den = ((rtc[0x05] & 0b00110000)>>4) *10 + (rtc[0x05] & 0b00001111)   # dny v mesici

  rok = ((rtc[0x05] & 0b11000000)>>6)    # rok je jen v rozsahu 0 az 3 
  rok = 2012 + rok   # tohle plati jen pro roky 2012 az 2016


  mes = ((rtc[0x06] & 0b00010000)>>4) *10 + (rtc[0x06] & 0b00001111)   # mesic
  dvt = ((rtc[0x06] & 0b11100000)>>5)                                  # den v tydnu : 1=Po  2=Ut ..... 7=Ne
   
  txtcas = str(hod) + ":" + str(min).rjust(2,"0") + ":"  + str(sek).rjust(2,"0") + "."  + str(ses).rjust(2,"0")
  txtdat = str(nazevdne[dvt]) + "  " +  str(den) + "." + str(mes) + ".(" + str(rok) + ")" 
  print "Cas v RTC:  " + txtdat + "  " + txtcas + " UTC" 

  
  if (nastav_cas == True):  # pokud je parametr tohoto podprogramu True, nastavi se RasPi podle RTC
    print "Cas v RasPi byl nastaven na:"
    prikaz="sudo date -u " + (str(mes).rjust(2,"0") +
                              str(den).rjust(2,"0") +
                              str(hod).rjust(2,"0") + 
                              str(min).rjust(2,"0") + 
                              str(rok) + "." + 
                              str(sek).rjust(2,"0"))
    os.system (prikaz)







#=============================================
# podprogram pro ulozeni systemoveho casu z RasPi do RTC obvodu
def rtc_zapis():
  datcas = datetime.datetime.utcnow()   # datcas bude obsahovat aktualni datum a cas v UTC (GMT)


  # setiny sekundy - registr 0x01
  ses = int((datcas.microsecond / 10000.0) + 0.5)
  seshi = int(ses/ 10)
  seslo = ses - (10 * seshi)
  bus.write_byte_data(adr,0x01,(seshi*16) + seslo)  


  # sekundy - registr 0x02
  sekhi = int(datcas.second / 10)
  seklo = datcas.second - (10 * sekhi)
  bus.write_byte_data(adr,0x02,(sekhi*16) + seklo)
  
  # minuty - registr 0x03
  minhi = int(datcas.minute / 10)
  minlo = datcas.minute - (10 * minhi)
  bus.write_byte_data(adr,0x03,(minhi*16) + minlo)
  
  # hodiny - registr 0x04
  hodhi = int(datcas.hour / 10)
  hodlo = datcas.hour - (10 * hodhi)
  bus.write_byte_data(adr,0x04,(hodhi*16) + hodlo)
  


  # rok + den - registr 0x05
  denhi = int(datcas.day / 10)
  denlo = datcas.day - (10 * denhi)
  mod4rok = datcas.year % 4   # rok bere jen jako 1 ze 4  (2012=0, 2013=1, 2014=2, 2015=3, 2016=0, 2017=1 ....)
  bus.write_byte_data(adr,0x05,(denhi*16) + denlo + (mod4rok*64))


  # den v tydnu + mesic - registr 0x06
  dvt = datetime.datetime.utcnow().isoweekday()  # Po=1 Ut=2 ....Ne=7
  meshi = int((datcas.month) / 10)
  meslo = datcas.month - (10 * meshi)
  bus.write_byte_data(adr,0x06, (meshi*16) + meslo + (dvt * 32)) 

  print "Systemovy cas v RasPi :" , datcas , "UTC"
  print "Systemovy cas byl prekopirovan do RTC obvodu."

 

#=============================================
# podprogram pro nastaveni casu v RasPi podle NTP serveru na internetu 
def ntp():   

  print "probiha restart NTP"
  prikaz="sudo /etc/init.d/ntp restart"
  os.system (prikaz)                         # spusteni prikazu v BASHi





#=============================================
#  ***** hlavni program *****
#=============================================



# vyhodnoceni predavaneho parametru pri spousteni programu
try:                      # odchytavani chyby, ktera by vznikla pri chybejicim parametru
  akce = sys.argv[1]      # parametr spousteneho programu do promenne "akce"
except:                   # kdyz parametr chybi, je nahrazen prazdnym retezcem
  akce=""

akce = akce.lower()       # prevod parametru na mala pismena

if (akce == "-i"):        # rozeskoky pri ruznych akcich
  rtc_info()
  
elif (akce == "-pi2rtc"):
  rtc_zapis()

elif (akce == "-rtc2pi"):
  rtc_info(True)

elif (akce == "-ntp2pi"):
  ntp()

else:                              # pri jakemkoli jinem parametru se zobrazi napoveda
  print "Pripustne parametry:"
  print "... -i        INFO - Jen zobrazi cas v RasPi a v RTC"
  print "... -pi2rtc   Zapise cas z RasPi do RTC"
  print "... -rtc2pi   Zapise cas z RTC do RasPi"
  print "... -ntp2pi   Restartuje NTP - seridi cas v RasPi podle internetu"
  



